@@ -18,42 +18,42 @@
 #include "packet.h"
 #include "rdata.h"
 
-static int 
+static int
 write_32(FILE *out, uint32_t val)
 {
 	val = htonl(val);
 	return write_data(out, &val, sizeof(val));
 }
 
-static int 
+static int
 write_16(FILE *out, uint16_t val)
 {
 	val = htons(val);
 	return write_data(out, &val, sizeof(val));
 }
 
-static int 
+static int
 write_8(FILE *out, uint8_t val)
 {
 	return write_data(out, &val, sizeof(val));
 }
 
-static int 
+static int
 write_str(FILE *out, const char* str)
 {
 	uint32_t len = strlen(str);
-	if(!write_32(out, len)) 
+	if(!write_32(out, len))
 		return 0;
 	return write_data(out, str, len);
 }
 
-void 
-diff_write_packet(const char* zone, uint32_t new_serial, uint16_t id, 
+void
+diff_write_packet(const char* zone, uint32_t new_serial, uint16_t id,
 	uint32_t seq_nr, uint8_t* data, size_t len, nsd_options_t* opt)
 {
 	const char* filename = opt->difffile;
 	FILE *df;
-	uint32_t file_len = sizeof(uint32_t) + strlen(zone) + 
+	uint32_t file_len = sizeof(uint32_t) + strlen(zone) +
 		sizeof(new_serial) + sizeof(id) + sizeof(seq_nr) + len;
 
 	df = fopen(filename, "a");
@@ -70,7 +70,7 @@ diff_write_packet(const char* zone, uint32_t new_serial, uint16_t id,
 		!write_16(df, id) ||
 		!write_32(df, seq_nr) ||
 		!write_data(df, data, len) ||
-		!write_32(df, file_len)) 
+		!write_32(df, file_len))
 	{
 		log_msg(LOG_ERR, "could not write to file %s: %s",
 			filename, strerror(errno));
@@ -79,10 +79,10 @@ diff_write_packet(const char* zone, uint32_t new_serial, uint16_t id,
 	fclose(df);
 }
 
-void 
+void
 diff_write_commit(const char* zone, uint32_t old_serial,
 	uint32_t new_serial, uint16_t id, uint32_t num_parts,
-	uint8_t commit, const char* log_str, 
+	uint8_t commit, const char* log_str,
 	nsd_options_t* opt)
 {
 	const char* filename = opt->difffile;
@@ -96,7 +96,7 @@ diff_write_commit(const char* zone, uint32_t old_serial,
 		return;
 	}
 
-	len = strlen(zone)+sizeof(len) + sizeof(old_serial) + 
+	len = strlen(zone)+sizeof(len) + sizeof(old_serial) +
 		sizeof(new_serial) + sizeof(id) + sizeof(num_parts) +
 		sizeof(commit) + strlen(log_str)+sizeof(len);
 
@@ -109,7 +109,7 @@ diff_write_commit(const char* zone, uint32_t old_serial,
 		!write_32(df, num_parts) ||
 		!write_8(df, commit) ||
 		!write_str(df, log_str) ||
-		!write_32(df, len)) 
+		!write_32(df, len))
 	{
 		log_msg(LOG_ERR, "could not write to file %s: %s",
 			filename, strerror(errno));
@@ -118,7 +118,7 @@ diff_write_commit(const char* zone, uint32_t old_serial,
 	fclose(df);
 }
 
-int 
+int
 db_crc_different(namedb_type* db)
 {
 	FILE *fd = fopen(db->filename, "r");
@@ -129,7 +129,7 @@ db_crc_different(namedb_type* db)
 			db->filename, strerror(errno));
 		return -1;
 	}
-	
+
 	/* seek to position of CRC, check it and magic no */
 	if(fseeko(fd, db->crc_pos, SEEK_SET)==-1) {
 		log_msg(LOG_ERR, "unable to fseeko %s: %s. db changed?",
@@ -166,7 +166,7 @@ db_crc_different(namedb_type* db)
 	return 1;
 }
 
-int 
+int
 diff_read_32(FILE *in, uint32_t* result)
 {
         if (fread(result, sizeof(*result), 1, in) == 1) {
@@ -177,7 +177,7 @@ diff_read_32(FILE *in, uint32_t* result)
         }
 }
 
-int 
+int
 diff_read_16(FILE *in, uint16_t* result)
 {
         if (fread(result, sizeof(*result), 1, in) == 1) {
@@ -188,7 +188,7 @@ diff_read_16(FILE *in, uint16_t* result)
         }
 }
 
-int 
+int
 diff_read_8(FILE *in, uint8_t* result)
 {
         if (fread(result, sizeof(*result), 1, in) == 1) {
@@ -198,15 +198,15 @@ diff_read_8(FILE *in, uint8_t* result)
         }
 }
 
-int 
+int
 diff_read_str(FILE* in, char* buf, size_t len)
 {
 	uint32_t disklen;
-	if(!diff_read_32(in, &disklen)) 
+	if(!diff_read_32(in, &disklen))
 		return 0;
-	if(disklen >= len) 
+	if(disklen >= len)
 		return 0;
-	if(fread(buf, disklen, 1, in) != 1) 
+	if(fread(buf, disklen, 1, in) != 1)
 		return 0;
 	buf[disklen] = 0;
 	return 1;
@@ -221,15 +221,15 @@ add_rdata_to_recyclebin(namedb_type* db, rr_type* rr)
 	{
 		if(!rdata_atom_is_domain(rr->type, i))
 			region_recycle(db->region, rr->rdatas[i].data,
-				rdata_atom_size(rr->rdatas[i]) 
+				rdata_atom_size(rr->rdatas[i])
 				+ sizeof(uint16_t));
 	}
-	region_recycle(db->region, rr->rdatas, 
+	region_recycle(db->region, rr->rdatas,
 		sizeof(rdata_atom_type)*rr->rdata_count);
 }
 
 /* this routine determines if below a domain there exist names with
- * data (is_existing) or no names below the domain have data. 
+ * data (is_existing) or no names below the domain have data.
  */
 static int
 has_data_below(domain_type* top)
@@ -246,7 +246,7 @@ has_data_below(domain_type* top)
 	return 0;
 }
 
-static void 
+static void
 rrset_delete(namedb_type* db, domain_type* domain, rrset_type* rrset)
 {
 	int i;
@@ -261,7 +261,7 @@ rrset_delete(namedb_type* db, domain_type* domain, rrset_type* rrset)
 	}
 	*pp = rrset->next;
 
-	DEBUG(DEBUG_XFRD,2, (LOG_INFO, "delete rrset of %s type %s", 
+	DEBUG(DEBUG_XFRD,2, (LOG_INFO, "delete rrset of %s type %s",
 		dname_to_string(domain_dname(domain),0),
 		rrtype_to_string(rrset_rrtype(rrset))));
 
@@ -286,7 +286,7 @@ rrset_delete(namedb_type* db, domain_type* domain, rrset_type* rrset)
 	/* recycle the memory space of the rrset */
 	for (i = 0; i < rrset->rr_count; ++i)
 		add_rdata_to_recyclebin(db, &rrset->rrs[i]);
-	region_recycle(db->region, rrset->rrs, 
+	region_recycle(db->region, rrset->rrs,
 		sizeof(rr_type) * rrset->rr_count);
 	region_recycle(db->region, rrset, sizeof(rrset_type));
 
@@ -307,20 +307,20 @@ rrset_delete(namedb_type* db, domain_type* domain, rrset_type* rrset)
 	rrset->rr_count = 0;
 }
 
-static int 
+static int
 rdatas_equal(rdata_atom_type *a, rdata_atom_type *b, int num, uint16_t type)
 {
 	int k;
 	for(k = 0; k < num; k++)
 	{
 		if(rdata_atom_is_domain(type, k)) {
-			/* check dname */
+			/* check dname: should compare case insensitive */
 			if(dname_compare(domain_dname(a[k].domain),
 				domain_dname(b[k].domain))!=0)
 				return 0;
 		} else {
 			/* check length */
-			if(a[k].data[0] != b[k].data[0]) 
+			if(a[k].data[0] != b[k].data[0])
 				return 0;
 			/* check data */
 			if(memcmp(a[k].data+1, b[k].data+1, a[k].data[0])!=0)
@@ -330,26 +330,29 @@ rdatas_equal(rdata_atom_type *a, rdata_atom_type *b, int num, uint16_t type)
 	return 1;
 }
 
-static int 
+static int
 find_rr_num(rrset_type* rrset,
-	uint16_t type, uint16_t klass, uint32_t ttl, 
+	uint16_t type, uint16_t klass,
 	rdata_atom_type *rdatas, ssize_t rdata_num)
 {
 	int i;
-	for(i=0; i<rrset->rr_count; ++i) {
-		if(rrset->rrs[i].ttl == ttl &&
-		   rrset->rrs[i].type == type &&
+
+	for(i=0; i < rrset->rr_count; ++i) {
+		if(rrset->rrs[i].type == type &&
 		   rrset->rrs[i].klass == klass &&
 		   rrset->rrs[i].rdata_count == rdata_num &&
 		   rdatas_equal(rdatas, rrset->rrs[i].rdatas, rdata_num, type))
+		{
 			return i;
+		}
 	}
+
 	return -1;
 }
 
-static int 
-delete_RR(namedb_type* db, const dname_type* dname, 
-	uint16_t type, uint16_t klass, uint32_t ttl, 
+static int
+delete_RR(namedb_type* db, const dname_type* dname,
+	uint16_t type, uint16_t klass,
 	buffer_type* packet, size_t rdatalen, zone_type *zone,
 	region_type* temp_region)
 {
@@ -357,14 +360,14 @@ delete_RR(namedb_type* db, const dname_type* dname,
 	rrset_type *rrset;
 	domain = domain_table_find(db->domains, dname);
 	if(!domain) {
-		log_msg(LOG_WARNING, "diff: domain %s does not exist", 
+		log_msg(LOG_WARNING, "diff: domain %s does not exist",
 			dname_to_string(dname,0));
 		buffer_skip(packet, rdatalen);
 		return 1; /* not fatal error */
 	}
 	rrset = domain_find_rrset(domain, zone, type);
 	if(!rrset) {
-		log_msg(LOG_WARNING, "diff: rrset %s does not exist", 
+		log_msg(LOG_WARNING, "diff: rrset %s does not exist",
 			dname_to_string(dname,0));
 		buffer_skip(packet, rdatalen);
 		return 1; /* not fatal error */
@@ -378,13 +381,13 @@ delete_RR(namedb_type* db, const dname_type* dname,
 		rdata_num = rdata_wireformat_to_rdata_atoms(
 			temp_region, temptable, type, rdatalen, packet, &rdatas);
 		if(rdata_num == -1) {
-			log_msg(LOG_ERR, "diff: bad rdata for %s", 
+			log_msg(LOG_ERR, "diff: bad rdata for %s",
 				dname_to_string(dname,0));
 			return 0;
 		}
-		rrnum = find_rr_num(rrset, type, klass, ttl, rdatas, rdata_num);
+		rrnum = find_rr_num(rrset, type, klass, rdatas, rdata_num);
 		if(rrnum == -1) {
-			log_msg(LOG_WARNING, "diff: RR %s does not exist", 
+			log_msg(LOG_WARNING, "diff: RR %s does not exist",
 				dname_to_string(dname,0));
 			return 1; /* not fatal error */
 		}
@@ -413,9 +416,9 @@ delete_RR(namedb_type* db, const dname_type* dname,
 	return 1;
 }
 
-static int 
-add_RR(namedb_type* db, const dname_type* dname, 
-	uint16_t type, uint16_t klass, uint32_t ttl, 
+static int
+add_RR(namedb_type* db, const dname_type* dname,
+	uint16_t type, uint16_t klass, uint32_t ttl,
 	buffer_type* packet, size_t rdatalen, zone_type *zone)
 {
 	domain_type* domain;
@@ -446,21 +449,21 @@ add_RR(namedb_type* db, const dname_type* dname,
 	rdata_num = rdata_wireformat_to_rdata_atoms(
 		db->region, db->domains, type, rdatalen, packet, &rdatas);
 	if(rdata_num == -1) {
-		log_msg(LOG_ERR, "diff: bad rdata for %s", 
+		log_msg(LOG_ERR, "diff: bad rdata for %s",
 			dname_to_string(dname,0));
 		return 0;
 	}
-	rrnum = find_rr_num(rrset, type, klass, ttl, rdatas, rdata_num);
+	rrnum = find_rr_num(rrset, type, klass, rdatas, rdata_num);
 	if(rrnum != -1) {
-		DEBUG(DEBUG_XFRD, 2, (LOG_ERR, "diff: RR %s already exists", 
+		DEBUG(DEBUG_XFRD, 2, (LOG_ERR, "diff: RR %s already exists",
 			dname_to_string(dname,0)));
 		/* ignore already existing RR: lenient accepting of messages */
 		return 1;
 	}
-	
+
 	/* re-alloc the rrs and add the new */
 	rrs_old = rrset->rrs;
-	rrset->rrs = region_alloc(db->region, 
+	rrset->rrs = region_alloc(db->region,
 		(rrset->rr_count+1) * sizeof(rr_type));
 	if(!rrset->rrs) {
 		log_msg(LOG_ERR, "out of memory, %s:%d", __FILE__, __LINE__);
@@ -486,20 +489,20 @@ add_RR(namedb_type* db, const dname_type* dname,
 			zone->updated = 1;
 			/* BUG #103 tweaked SOA ttl value */
 			if(zone->soa_nx_rrset == 0) {
-				zone->soa_nx_rrset = region_alloc(db->region, 
+				zone->soa_nx_rrset = region_alloc(db->region,
 					sizeof(rrset_type));
 				if(!zone->soa_nx_rrset) {
-					log_msg(LOG_ERR, "out of memory, %s:%d", 
+					log_msg(LOG_ERR, "out of memory, %s:%d",
 						__FILE__, __LINE__);
 					exit(1);
 				}
 				zone->soa_nx_rrset->rr_count = 1;
 				zone->soa_nx_rrset->next = 0;
 				zone->soa_nx_rrset->zone = zone;
-				zone->soa_nx_rrset->rrs = region_alloc(db->region, 
+				zone->soa_nx_rrset->rrs = region_alloc(db->region,
 					sizeof(rr_type));
 				if(!zone->soa_nx_rrset->rrs) {
-					log_msg(LOG_ERR, "out of memory, %s:%d", 
+					log_msg(LOG_ERR, "out of memory, %s:%d",
 						__FILE__, __LINE__);
 					exit(1);
 				}
@@ -529,7 +532,7 @@ add_RR(namedb_type* db, const dname_type* dname,
 	return 1;
 }
 
-static zone_type* 
+static zone_type*
 find_zone(namedb_type* db, const dname_type* zone_name, nsd_options_t* opt,
 	size_t child_count)
 {
@@ -578,7 +581,7 @@ find_zone(namedb_type* db, const dname_type* zone_name, nsd_options_t* opt,
 		exit(1);
 	}
 	memset(zone->dirty, 0, sizeof(uint8_t)*child_count);
-	zone->opts = zone_options_find(opt, domain_dname(zone->apex)); 
+	zone->opts = zone_options_find(opt, domain_dname(zone->apex));
 	if(!zone->opts) {
 		log_msg(LOG_ERR, "xfr: zone %s not in config.",
 			dname_to_string(zone_name,0));
@@ -591,7 +594,7 @@ find_zone(namedb_type* db, const dname_type* zone_name, nsd_options_t* opt,
 	return zone;
 }
 
-static void 
+static void
 delete_zone_rrs(namedb_type* db, zone_type* zone)
 {
 	rrset_type *rrset;
@@ -609,7 +612,7 @@ delete_zone_rrs(namedb_type* db, zone_type* zone)
 		domain = domain_next(domain);
 	}
 
-	DEBUG(DEBUG_XFRD, 1, (LOG_INFO, "axfrdel: recyclebin holds %lu bytes", 
+	DEBUG(DEBUG_XFRD, 1, (LOG_INFO, "axfrdel: recyclebin holds %lu bytes",
 		(unsigned long) region_get_recycle_size(db->region)));
 #ifndef NDEBUG
 	if(nsd_debug_level >= 1)
@@ -623,8 +626,8 @@ delete_zone_rrs(namedb_type* db, zone_type* zone)
 	assert(zone->updated == 1);
 }
 
-static int 
-apply_ixfr(namedb_type* db, FILE *in, const off_t* startpos, 
+static int
+apply_ixfr(namedb_type* db, FILE *in, const off_t* startpos,
 	const char* zone, uint32_t serialno, nsd_options_t* opt,
 	uint16_t id, uint32_t seq_nr, uint32_t seq_total,
 	int* is_axfr, int* delete_mode, int* rr_count,
@@ -641,8 +644,10 @@ apply_ixfr(namedb_type* db, FILE *in, const off_t* startpos,
 	char file_zone_name[3072];
 	uint32_t file_serial, file_seq_nr;
 	uint16_t file_id;
+	off_t mempos;
 
-	if(fseeko(in, *startpos, SEEK_SET) == -1) {
+	memmove(&mempos, startpos, sizeof(off_t));
+	if(fseeko(in, mempos, SEEK_SET) == -1) {
 		log_msg(LOG_INFO, "could not fseeko: %s.", strerror(errno));
 		return 0;
 	}
@@ -675,12 +680,14 @@ apply_ixfr(namedb_type* db, FILE *in, const off_t* startpos,
 		!diff_read_32(in, &file_seq_nr))
 	{
 		log_msg(LOG_ERR, "could not part data");
+		region_destroy(region);
 		return 0;
 	}
-	
+
 	if(strcmp(file_zone_name, zone) != 0 || serialno != file_serial ||
 		id != file_id || seq_nr != file_seq_nr) {
 		log_msg(LOG_ERR, "internal error: reading part with changed id");
+		region_destroy(region);
 		return 0;
 	}
 	msglen = filelen - sizeof(uint32_t)*3 - sizeof(uint16_t)
@@ -693,7 +700,7 @@ apply_ixfr(namedb_type* db, FILE *in, const off_t* startpos,
 		region_destroy(region);
 		return 0;
 	}
-	
+
 	if(msglen > QIOBUFSZ) {
 		log_msg(LOG_ERR, "msg too long");
 		region_destroy(region);
@@ -707,7 +714,7 @@ apply_ixfr(namedb_type* db, FILE *in, const off_t* startpos,
 	}
 	buffer_set_limit(packet, msglen);
 
-	/* only answer section is really used, question, additional and 
+	/* only answer section is really used, question, additional and
 	   authority section RRs are skipped */
 	qcount = QDCOUNT(packet);
 	ancount = ANCOUNT(packet);
@@ -772,6 +779,7 @@ apply_ixfr(namedb_type* db, FILE *in, const off_t* startpos,
 		*rr_count = 1;
 		*is_axfr = 0;
 		*delete_mode = 0;
+
 		DEBUG(DEBUG_XFRD,2, (LOG_INFO, "diff: %s start count %d, ax %d, delmode %d",
 			dname_to_string(dname_zone, 0), *rr_count, *is_axfr, *delete_mode));
 	}
@@ -797,7 +805,7 @@ apply_ixfr(namedb_type* db, FILE *in, const off_t* startpos,
 		ttl = buffer_read_u32(packet);
 		rrlen = buffer_read_u16(packet);
 		if(!buffer_available(packet, rrlen)) {
-			log_msg(LOG_ERR, "bad xfr RR rdata %d, len %d have %d", 
+			log_msg(LOG_ERR, "bad xfr RR rdata %d, len %d have %d",
 				*rr_count, rrlen, (int)buffer_remaining(packet));
 			region_destroy(region);
 			return 0;
@@ -848,31 +856,37 @@ apply_ixfr(namedb_type* db, FILE *in, const off_t* startpos,
 			buffer_skip(packet, rrlen);
 			continue;
 		}
-		DEBUG(DEBUG_XFRD,2, (LOG_INFO, "xfr %s RR dname is %s type %s", 
+
+		DEBUG(DEBUG_XFRD,2, (LOG_INFO, "xfr %s RR dname is %s type %s",
 			*delete_mode?"del":"add",
 			dname_to_string(dname,0), rrtype_to_string(type)));
 		if(*delete_mode) {
 			/* delete this rr */
 			if(!*is_axfr && type == TYPE_SOA && counter==ancount-1
-				&& seq_nr == seq_total-1)
+				&& seq_nr == seq_total-1) {
 				continue; /* do not delete final SOA RR for IXFR */
-			if(!delete_RR(db, dname, type, klass, ttl, packet, 
-				rrlen, zone_db, region))
+			}
+			if(!delete_RR(db, dname, type, klass, packet,
+				rrlen, zone_db, region)) {
+				region_destroy(region);
 				return 0;
+			}
 		}
 		else
 		{
 			/* add this rr */
-			if(!add_RR(db, dname, type, klass, ttl, packet, 
-				rrlen, zone_db))
+			if(!add_RR(db, dname, type, klass, ttl, packet,
+				rrlen, zone_db)) {
+				region_destroy(region);
 				return 0;
+			}
 		}
 	}
 	region_destroy(region);
 	return 1;
 }
 
-static int 
+static int
 check_for_bad_serial(namedb_type* db, const char* zone_str, uint32_t old_serial)
 {
 	/* see if serial OK with in-memory serial */
@@ -886,7 +900,7 @@ check_for_bad_serial(namedb_type* db, const char* zone_str, uint32_t old_serial)
 	if(zone && zone->apex == domain && zone->soa_rrset && old_serial)
 	{
 		uint32_t memserial;
-		memcpy(&memserial, rdata_atom_data(zone->soa_rrset->rrs[0].rdatas[2]), 
+		memcpy(&memserial, rdata_atom_data(zone->soa_rrset->rrs[0].rdatas[2]),
 			sizeof(uint32_t));
 		if(old_serial != ntohl(memserial)) {
 			region_destroy(region);
@@ -926,7 +940,7 @@ struct diff_xfrpart {
 	off_t file_pos;
 };
 
-static struct diff_read_data* 
+static struct diff_read_data*
 diff_read_data_create()
 {
 	region_type* region = region_create(xalloc, free);
@@ -937,7 +951,7 @@ diff_read_data_create()
 		exit(1);
 	}
 	data->region = region;
-	data->zones = rbtree_create(region, 
+	data->zones = rbtree_create(region,
 		(int (*)(const void *, const void *)) dname_compare);
 	return data;
 }
@@ -953,9 +967,9 @@ diff_read_find_zone(struct diff_read_data* data, const char* name)
 
 static int intcompf(const void* a, const void* b)
 {
-	if(*(uint32_t*)a < *(uint32_t*)b) 
+	if(*(uint32_t*)a < *(uint32_t*)b)
 		return -1;
-	if(*(uint32_t*)a > *(uint32_t*)b) 
+	if(*(uint32_t*)a > *(uint32_t*)b)
 		return +1;
 	return 0;
 }
@@ -1024,8 +1038,8 @@ mark_and_exit(nsd_options_t* opt, FILE* f, off_t commitpos, const char* desc)
 	exit(1);
 }
 
-static int 
-read_sure_part(namedb_type* db, FILE *in, nsd_options_t* opt, 
+static int
+read_sure_part(namedb_type* db, FILE *in, nsd_options_t* opt,
 	struct diff_read_data* data, struct diff_log** log,
 	size_t child_count)
 {
@@ -1083,7 +1097,7 @@ read_sure_part(namedb_type* db, FILE *in, nsd_options_t* opt,
 		return 1;
 	}
 	if(committed && check_for_bad_serial(db, zone_buf, old_serial)) {
-		DEBUG(DEBUG_XFRD,1, (LOG_ERR, 
+		DEBUG(DEBUG_XFRD,1, (LOG_ERR,
 			"skipping diff file commit with bad serial"));
 		zp->parts->root = RBTREE_NULL;
 		zp->parts->count = 0;
@@ -1098,7 +1112,7 @@ read_sure_part(namedb_type* db, FILE *in, nsd_options_t* opt,
 		}
 	}
 	if(!have_all_parts) {
-		DEBUG(DEBUG_XFRD,1, (LOG_ERR, 
+		DEBUG(DEBUG_XFRD,1, (LOG_ERR,
 			"skipping diff file commit without all parts"));
 		if(thislog)
 			thislog->error = "error missing parts";
@@ -1121,7 +1135,8 @@ read_sure_part(namedb_type* db, FILE *in, nsd_options_t* opt,
 			if(!apply_ixfr(db, in, &xp->file_pos, zone_buf, new_serial, opt,
 				id, xp->seq_nr, num_parts, &is_axfr, &delete_mode,
 				&rr_count, child_count)) {
-				log_msg(LOG_ERR, "bad ixfr packet part %d", (int)i);
+				log_msg(LOG_ERR, "bad ixfr packet part %d in %s", (int)i,
+					opt->difffile);
 				mark_and_exit(opt, in, commitpos, log_buf);
 			}
 		}
@@ -1133,7 +1148,7 @@ read_sure_part(namedb_type* db, FILE *in, nsd_options_t* opt,
 	else {
 	 	DEBUG(DEBUG_XFRD,1, (LOG_INFO, "skipping xfr: %s", log_buf));
 	}
-	
+
 	/* clean out the parts for the zone after the commit/rollback */
 	zp->parts->root = RBTREE_NULL;
 	zp->parts->count = 0;
@@ -1176,7 +1191,7 @@ store_ixfr_data(FILE *in, uint32_t len, struct diff_read_data* data, off_t* star
 	return 1;
 }
 
-static int 
+static int
 read_process_part(namedb_type* db, FILE *in, uint32_t type,
 	nsd_options_t* opt, struct diff_read_data* data,
 	struct diff_log** log, size_t child_count, off_t* startpos)
@@ -1192,7 +1207,7 @@ read_process_part(namedb_type* db, FILE *in, uint32_t type,
 	}
 	else if(type == DIFF_PART_SURE) {
 		DEBUG(DEBUG_XFRD,2, (LOG_INFO, "part SURE len %d", len));
-		if(!read_sure_part(db, in, opt, data, log, child_count)) 
+		if(!read_sure_part(db, in, opt, data, log, child_count))
 			return 0;
 	} else {
 		DEBUG(DEBUG_XFRD,1, (LOG_INFO, "unknown part %x len %d", type, len));
@@ -1215,6 +1230,8 @@ find_smallest_offset(struct diff_read_data* data, off_t* offset)
 	int found_any = 0;
 	struct diff_zone* dz;
 	struct diff_xfrpart* dx;
+	off_t mem_offset, mem_fpos;
+
 	if(!data || !data->zones)
 		return 0;
 	RBTREE_FOR(dz, struct diff_zone*, data->zones)
@@ -1223,12 +1240,16 @@ find_smallest_offset(struct diff_read_data* data, off_t* offset)
 			continue;
 		RBTREE_FOR(dx, struct diff_xfrpart*, dz->parts)
 		{
+			memmove(&mem_fpos, &dx->file_pos, sizeof(off_t));
+
 			if(found_any) {
-				if(dx->file_pos < *offset)
-					memmove(offset, &dx->file_pos, sizeof(off_t));
+				memmove(&mem_offset, offset, sizeof(off_t));
+
+				if(mem_fpos < mem_offset)
+					memmove(offset, &mem_fpos, sizeof(off_t));
 			} else {
 				found_any = 1;
-				memmove(offset, &dx->file_pos, sizeof(off_t));
+				memmove(offset, &mem_fpos, sizeof(off_t));
 			}
 		}
 	}
@@ -1236,7 +1257,7 @@ find_smallest_offset(struct diff_read_data* data, off_t* offset)
 	return found_any;
 }
 
-int 
+int
 diff_read_file(namedb_type* db, nsd_options_t* opt, struct diff_log** log,
 	size_t child_count)
 {
@@ -1264,13 +1285,14 @@ diff_read_file(namedb_type* db, nsd_options_t* opt, struct diff_log** log,
 	startpos = ftello(df);
 	if(startpos == -1) {
 		log_msg(LOG_INFO, "could not ftello: %s.", strerror(errno));
+		region_destroy(data->region);
 		return 0;
 	}
 
-	while(diff_read_32(df, &type)) 
+	while(diff_read_32(df, &type))
 	{
 		DEBUG(DEBUG_XFRD,2, (LOG_INFO, "iter loop"));
-		if(!read_process_part(db, df, type, opt, data, log, 
+		if(!read_process_part(db, df, type, opt, data, log,
 			child_count, &startpos))
 		{
 			log_msg(LOG_INFO, "error processing diff file");
@@ -1280,6 +1302,7 @@ diff_read_file(namedb_type* db, nsd_options_t* opt, struct diff_log** log,
 		startpos = ftello(df);
 		if(startpos == -1) {
 			log_msg(LOG_INFO, "could not ftello: %s.", strerror(errno));
+			region_destroy(data->region);
 			return 0;
 		}
 	}
@@ -1298,7 +1321,7 @@ diff_read_file(namedb_type* db, nsd_options_t* opt, struct diff_log** log,
 			db->diff_skip = 0;
 		}
 	}
-	
+
 	region_destroy(data->region);
 	fclose(df);
 	return 1;
